C----------------------------------------------------------------- 
C Sample driver program for IFFCO.
C Users can modify this file to run IFFCO on their own problems.
C This sample driver sets each of the IFFCO parameters in the source
C code.  Another file, main.f, gives an example of reading the 
C parameters from a file.
C----------------------------------------------------------------- 

      program main

      implicit none

C Declare local variables
      integer i

C Declare IFFCO global variables.  These contain information about
C the result of the run.
      integer nevalsIF
      double precision fminIF, xminIF(24), mepsIF, fmaxIF
      common /globalIF/nevalsIF
      common /globalIF2/mepsIF,fminIF,xminIF,fmaxIF


C Declare IFFCO parameters.
      integer dim
C     Define the dimension of the problem here
      parameter(dim = 2)

      external myfunc
      double precision x(dim), u(dim), l(dim), fscale, minh, maxh
      double precision termTol, f
      integer n, maxit(2), restart, writ, maxcuts, option(7)

      integer ilen, dlen, clen
C     Define the size of any extra data to be passed here
      parameter(ilen = 1, dlen = 1, clen = 1)
      integer idata(ilen)
      double precision ddata(dlen)
      character cdata(clen)


C     Variables and function declarations for parallel functionality
      external comminitIF, commexitIF, getmytidIF
      integer mytid, mastertid, nprocs

C     Initialize MPI or PVM
      call comminitIF()

C     This information will help determine who is the master processor later 
      call getmytidIF(mytid)
      call gettidIF(0, mastertid)


C     Initialize IFFCO parameters
C     Initial point
C     Note:  if you change this, also set option(3) = 0.
      do 5 i = 1,dim
        x(i) = 0.0
 5    continue

C     Upper and lower bounds
      do 10 i = 1,dim
        u(i) = 1.0
        l(i) = 0.0
 10   continue

C     Scaling
      fscale = 0

C     Smallest and largest scales
      minh = 2.0**(-4)
      maxh = 0.50D0

C     Max iterations on a given scale and for entire algorithm
      maxit(1) = 0
      maxit(2) = 0

C     Number of restarts to do
      restart = 0

C     Level of output
      writ = 14

C     Termination tolerance
      termTol = 0.0

C     Maximum cuts to take in a linesearch
      maxcuts = 0

C     Options
      option(1) = 1
      option(2) = 3
      option(3) = 1
      option(4) = 1
      option(5) = 0
      option(6) = 0
      option(7) = 0

C     Additional data (if applicable)
      idata(1) = 0
      ddata(1) = 0
      cdata(1) = '0' 

C     Start IFFCO
      call iffco(myfunc,x,u,l,fscale,minh,maxh,dim,maxit,
     *     restart,writ,termTol,f,maxcuts,option, 
     *     idata, ilen, ddata, dlen, cdata, clen)

C     Print brief results of the run.

C     (only the master processor does this for parallel versions)
      if(mytid.eq.mastertid) then

      write(*,*) 'IFFCO found this minimum:'
      write(*,*) 'f = ', fminIF
      do 20 i = 1,n
        write(*,*) 'x(', i, ') = ', x(i)
 20   continue
      write(*,*) 'IFFCO used ', nevalsIF, ' function evaluations.'

      end if

C     Exit MPI or PVM
      call commexitIF()

      end
