C----------------------------------------------------------------- 
C Test problems for optimization.
C The subroutines in this file were written by Joerg Gablonsky.
C teconst, telinear, and tequadr are straightforward constant, linear,
C and quadratic functions, respectively.
C branin, shekel, hartman, and goldprice are from
C     EDITOR = "L.C.W. Dixon and G.P. Szeg{\"{o}}",
C     AUTHOR = "L.C.W. Dixon and G.P. Szeg{\"{o}}",
C     BOOKTITLE = "Towards {G}lobal {O}ptimization 2",
C     TITLE = "The {G}lobal {O}ptimisation {P}roblem: {A}n {I}ntroduction",
C     VOLUME = 2,
C     PAGES = "1-15",
C     PUBLISHER = "North-{H}olland {P}ublishing {C}ompany",
C     YEAR = "1978",
C sixhump and shubert are from
C     AUTHOR = "Yong Yao",
C     TITLE = "Dynamic {T}unneling {A}lgorithm for {G}lobal {O}ptimization",
C     JOURNAL = "IEEE {T}ransactions on {S}ystems, {M}an, and {C}ybernetics",
C     VOLUME = 19,
C     NUMBER = 5,
C     MONTH = "September / {O}ctober",
C     YEAR = 1989
C----------------------------------------------------------------- 

      subroutine myfunc(n, x, f, flag,
     +                  iidata, iisize, ddata, idsize, cdata, icsize)

      implicit none
      integer n, flag
      double precision x(n)
      double precision f
      integer problem
C+-----------------------------------------------------------------------+
C| Variables to pass user defined data to the function to be optimized.  |
C+-----------------------------------------------------------------------+
      INTEGER iisize, idsize, icsize
      INTEGER iidata(iisize)
      Double Precision ddata(idsize)
      Character*40 cdata(icsize)

      problem = iidata(iisize)
      if (problem .eq. 0) then
         CALL teconst(x, n, flag, f, iidata, 
     +        iisize, ddata, idsize, cdata, icsize)
      else if (problem .eq. 1) then 
         CALL telinear(x, n, flag, f, iidata, 
     +        iisize, ddata, idsize, cdata, icsize)
      else if (problem .eq. 2) then 
         CALL tequadr(x, n, flag, f, iidata, 
     +        iisize, ddata, idsize, cdata, icsize)
      else if (problem .eq. 3) then 
         CALL branin(x, n, flag, f, iidata, 
     +        iisize, ddata, idsize, cdata, icsize)
      else if ((problem .ge. 4) .and. (problem .LE. 6)) then 
         CALL shekel(x, n, flag, f, iidata, 
     +        iisize, ddata, idsize, cdata, icsize)
      else if ((problem .ge. 7) .and. (problem .LE. 8)) then 
         CALL hartman(x, n, flag, f, iidata, 
     +        iisize, ddata, idsize, cdata, icsize)
      else if (problem .eq. 9) then 
         CALL goldprice(x, n, flag, f, iidata, 
     +        iisize, ddata, idsize, cdata, icsize)
      else if (problem .eq. 10) then 
         CALL sixhump(x, n, flag, f, iidata, 
     +        iisize, ddata, idsize, cdata, icsize)
      else if (problem .eq. 11) then 
         CALL shubert(x, n, flag, f, iidata, 
     +        iisize, ddata, idsize, cdata, icsize)
      end if
      end

      subroutine tequadr(x,n,flag,f,
     +                  iidata, iisize, ddata, idsize, cdata, icsize)

      implicit none
      integer n,flag,i
      double precision x(n)
      double precision f
C+-----------------------------------------------------------------------+
C| Variables to pass user defined data to the function to be optimized.  |
C+-----------------------------------------------------------------------+
      INTEGER iisize, idsize, icsize
      INTEGER iidata(iisize)
      Double Precision ddata(idsize)
      Character*40 cdata(icsize)

      f = 10
      do 100, i = 1,n
         f = f + (x(i)-5.3)*(x(i)-5.3)
100   continue
      flag = 0
      end

      subroutine teconst(x,n,flag,f,
     +                  iidata, iisize, ddata, idsize, cdata, icsize)

      implicit none
      integer n,flag
      double precision x(n)
      double precision f
C+-----------------------------------------------------------------------+
C| Variables to pass user defined data to the function to be optimized.  |
C+-----------------------------------------------------------------------+
      INTEGER iisize, idsize, icsize
      INTEGER iidata(iisize)
      Double Precision ddata(idsize)
      Character*40 cdata(icsize)

      f = 100
      flag = 0
      end

      subroutine telinear(x,n,flag,f,
     +                  iidata, iisize, ddata, idsize, cdata, icsize)

      implicit none
      integer n,flag,i
      double precision x(n)
      double precision f
C+-----------------------------------------------------------------------+
C| Variables to pass user defined data to the function to be optimized.  |
C+-----------------------------------------------------------------------+
      INTEGER iisize, idsize, icsize
      INTEGER iidata(iisize)
      Double Precision ddata(idsize)
      Character*40 cdata(icsize)

      f = 0.D0
      f = 2.D0*x(1)
      do 100, i = 2,n
         f = f + (x(i))*3.D0
100   continue
      flag = 0
      end


      subroutine goldprice(x,n,flag,f,
     +                  iidata, iisize, ddata, idsize, cdata, icsize)
      implicit none
      integer n,flag
      double precision x(n)
      double precision f, help1, help2, pi
C+-----------------------------------------------------------------------+
C| Variables to pass user defined data to the function to be optimized.  |
C+-----------------------------------------------------------------------+
      INTEGER iisize, idsize, icsize
      INTEGER iidata(iisize)
      Double Precision ddata(idsize)
      Character*40 cdata(icsize)

      f = 0.D0
      help1 = 0.D0
      pi = acos(0.0D0)
      pi = pi + pi
      help1 = x(1)+x(2)+1.D0
      f = 19.D0-14.D0*(x(1)+x(2))+3*(x(1)*x(1)+x(2)*x(2))+6*x(1)*x(2)
      help1 = help1*help1*f+1
      help2 = 2.D0*x(1)-3.D0*x(2)
      f = 18-32*x(1)+12*x(1)*x(1) +48*x(2)-36*x(1)*x(2)+27*x(2)*x(2)
      help2 = help2*help2*f + 30.D0
      f = help1*help2
      flag = 0
      end


      subroutine branin(x,n,flag,f,
     +                  iidata, iisize, ddata, idsize, cdata, icsize)
      implicit none
      integer n,flag
      double precision x(n)
      double precision f, help1, pi
C+-----------------------------------------------------------------------+
C| Variables to pass user defined data to the function to be optimized.  |
C+-----------------------------------------------------------------------+
      INTEGER iisize, idsize, icsize
      INTEGER iidata(iisize)
      Double Precision ddata(idsize)
      Character*40 cdata(icsize)

      f = 0.D0
      help1 = 0.D0
      pi = acos(0.0D0)
      pi = pi + pi
      help1 = x(2) - 1.275D0*x(1)*x(1)/(pi*pi)
      help1 = help1 + 5*x(1)/pi - 6.D0
      f = help1*help1 + 10.D0 + (1 - 1/(8.D0*pi))*10.D0*cos(x(1))
      flag = 0
      end

      subroutine shekel(x,n,flag,f,
     +                  iidata, iisize, ddata, idsize, cdata, icsize)
      implicit none
      integer n,flag,i,j,m
      double precision x(n)
      double precision f, help1, pi
      double precision a(10,4), c(10)
C+-----------------------------------------------------------------------+
C| Variables to pass user defined data to the function to be optimized.  |
C+-----------------------------------------------------------------------+
      INTEGER iisize, idsize, icsize
      INTEGER iidata(iisize)
      Double Precision ddata(idsize)
      Character*40 cdata(icsize)

      m = iidata(1)
      f = 0.D0
      help1 = 0.D0
      pi = acos(0.0D0)
      pi = pi + pi
      a(1,1) = 4.0D0
      a(1,2) = 4.0D0
      a(1,3) = 4.0D0
      a(1,4) = 4.0D0

      a(2,1) = 1.0D0
      a(2,2) = 1.0D0
      a(2,3) = 1.0D0
      a(2,4) = 1.0D0

      a(3,1) = 8.0D0
      a(3,2) = 8.0D0
      a(3,3) = 8.0D0
      a(3,4) = 8.0D0

      a(4,1) = 6.0D0
      a(4,2) = 6.0D0
      a(4,3) = 6.0D0
      a(4,4) = 6.0D0

      a(5,1) = 3.0D0
      a(5,2) = 7.0D0
      a(5,3) = 3.0D0
      a(5,4) = 7.0D0

      a(6,1) = 2.0D0
      a(6,2) = 9.0D0
      a(6,3) = 2.0D0
      a(6,4) = 9.0D0

      a(7,1) = 5.0D0
      a(7,2) = 5.0D0
      a(7,3) = 3.0D0
      a(7,4) = 3.0D0

      a(8,1) = 8.0D0
      a(8,2) = 1.0D0
      a(8,3) = 8.0D0
      a(8,4) = 1.0D0

      a(9,1) = 6.0D0
      a(9,2) = 2.0D0
      a(9,3) = 6.0D0
      a(9,4) = 2.0D0

      a(10,1) = 7.0D0
      a(10,2) = 3.6D0
      a(10,3) = 7.0D0
      a(10,4) = 3.6D0

      c(1)  = .1D0
      c(2)  = .2D0
      c(3)  = .2D0
      c(4)  = .4D0
      c(5)  = .4D0
      c(6)  = .6D0
      c(7)  = .3D0
      c(8)  = .7D0
      c(9)  = .5D0
      c(10) = .5D0
      f = 0.D0

      DO 10, i=1,m
        help1 = 0.D0
        DO 20, j = 1,n
          help1 = help1 + (x(j) - a(i,j))*(x(j) - a(i,j))
20      continue
        help1 = help1 + c(i)
        f = f - 1/help1
10    continue

      flag = 0
      end

      subroutine hartman(x,n,flag,f,
     +                  iidata, iisize, ddata, idsize, cdata, icsize)
      implicit none
      integer n,flag,i,j,m
      double precision x(n)
      double precision f, help1, pi
      double precision a(4,6), c(4), p(4,6)
C+-----------------------------------------------------------------------+
C| Variables to pass user defined data to the function to be optimized.  |
C+-----------------------------------------------------------------------+
      INTEGER iisize, idsize, icsize
      INTEGER iidata(iisize)
      Double Precision ddata(idsize)
      Character*40 cdata(icsize)


      m = iidata(1)
      f = 0.D0
      help1 = 0.D0
      pi = acos(0.0D0)
      pi = pi + pi
      if (n .eq. 3) THEN
        a(1,1) =  3.0D0
        a(1,2) = 10.0D0
        a(1,3) = 30.0D0

        a(2,1) =   .1D0
        a(2,2) = 10.0D0
        a(2,3) = 35.0D0

        a(3,1) =  3.0D0
        a(3,2) = 10.0D0
        a(3,3) = 30.0D0

        a(4,1) =  0.1D0
        a(4,2) = 10.0D0
        a(4,3) = 35.0D0

        c(1)  = 1.0D0
        c(2)  = 1.2D0
        c(3)  = 3.0D0
        c(4)  = 3.2D0

        p(1,1) =  0.3689D0
        p(1,2) =  0.1170D0
        p(1,3) =  0.2673D0

        p(2,1) =  0.4699D0
        p(2,2) =  0.4387D0
        p(2,3) =  0.7470D0

        p(3,1) =  0.1091D0
        p(3,2) =  0.8732D0
        p(3,3) =  0.5547D0

        p(4,1) =  0.03815D0
        p(4,2) =  0.5743D0
        p(4,3) =  0.8828D0
      ELSE
        a(1,1) = 10.0D0
        a(1,2) =  3.0D0
        a(1,3) = 17.0D0
        a(1,4) =  3.5D0
        a(1,5) =  1.7D0
        a(1,6) =  8.0D0

        a(2,1) =  0.05D0
        a(2,2) = 10.0D0
        a(2,3) = 17.0D0
        a(2,4) =  0.1D0
        a(2,5) =  8.0D0
        a(2,6) = 14.0D0

        a(3,1) =  3.0D0
        a(3,2) =  3.5D0
        a(3,3) =  1.7D0
        a(3,4) = 10.0D0
        a(3,5) = 17.0D0
        a(3,6) =  8.0D0

        a(4,1) = 17.0D0
        a(4,2) =  8.0D0
        a(4,3) =  0.05D0
        a(4,4) = 10.0D0
        a(4,5) =  0.1D0
        a(4,6) = 14.0D0

        c(1)  = 1.0D0
        c(2)  = 1.2D0
        c(3)  = 3.0D0
        c(4)  = 3.2D0

        p(1,1) = .1312D0
        p(1,2) = .1696D0
        p(1,3) = .5569D0
        p(1,4) = .0124D0
        p(1,5) = .8283D0
        p(1,6) = .5886D0

        p(2,1) = .2329D0
        p(2,2) = .4135D0
        p(2,3) = .8307D0
        p(2,4) = .3736D0
        p(2,5) = .1004D0
        p(2,6) = .9991D0

        p(3,1) = .2348D0
        p(3,2) = .1451D0
        p(3,3) = .3522D0
        p(3,4) = .2883D0
        p(3,5) = .3047D0
        p(3,6) = .6650D0

        p(4,1) = .4047D0
        p(4,2) = .8828D0
        p(4,3) = .8732D0
        p(4,4) = .5743D0
        p(4,5) = .1091D0
        p(4,6) = .0381D0

      END IF
      f = 0.D0

      DO 10, i=1,m
        help1 = 0.D0
        DO 20, j = 1,n
          help1 = help1 + a(i,j)*(x(j) - p(i,j))*(x(j) - p(i,j))
20      continue
        help1 = c(i)*dexp(-help1)
        f = f - help1
10    continue

      flag = 0
      end


      subroutine sixhump(x,n,flag,f,
     +                  iidata, iisize, ddata, idsize, cdata, icsize)
      implicit none
      integer n,flag
      double precision x(n)
      double precision f, help1, help2, pi
C+-----------------------------------------------------------------------+
C| Variables to pass user defined data to the function to be optimized.  |
C+-----------------------------------------------------------------------+
      INTEGER iisize, idsize, icsize
      INTEGER iidata(iisize)
      Double Precision ddata(idsize)
      Character*40 cdata(icsize)

      f = 0.D0
      help1 = 0.D0
      pi = acos(0.0D0)
      pi = pi + pi
      help1 = 4.D0 - 2.1D0*x(1)*x(1)+x(1)*x(1)*x(1)*x(1)/3.D0
      help2 = -4.D0 + 4.D0*x(2)*x(2)
      f = help1*x(1)*x(1) + x(1)*x(2) + help2*x(2)*x(2)
      end

      subroutine shubert(x,n,flag,f,
     +                  iidata, iisize, ddata, idsize, cdata, icsize)
      implicit none
      integer n,flag,i
      double precision x(n)
      double precision f, help1, help2
C+-----------------------------------------------------------------------+
C| Variables to pass user defined data to the function to be optimized.  |
C+-----------------------------------------------------------------------+
      INTEGER iisize, idsize, icsize
      INTEGER iidata(iisize)
      Double Precision ddata(idsize)
      Character*40 cdata(icsize)

      f = 0.D0
      help1 = 0.D0
      help2 = 0.D0
      do 10, i = 1,5
        help1 = help1+real(i)*dcos((real(i)+1.D0)*x(1) + real(i))
        help2 = help2+real(i)*dcos((real(i)+1.D0)*x(2) + real(i))
10    CONTINUE
      f = help1*help2
      flag = 0
      end

